/**********************************************************************************
 * ClassName  :   InterfaceLoader
 * Date       :   17, May  1998
 * Autor      :   Remus Pereni [RP]
 * Description:   Vezi mai jos
 **********************************************************************************/

package JaRPE;

import java.io.*;
import java.util.zip.*;
import java.util.Vector;
import java.awt.Frame;


/**
*   Clasa care asigura serializarea tuturor obiectelor in mod unitar
* intr-un fisier ZIP, necompresat. Idea este ca o interfata are mai
* multe elemente care serializate ar fi mai multe fisiere, in acest
* mod (prin zipare) desi fiecare obiect consista dintr-un fisier
* independent din exterior avem de lucru numai cu un singur fisier
* , arhiva care contine toate elementele serializate ale interfatei.
*
* @author   Remus Pereni
* @version  0.01, 17 May 1998 02:48 PM
* @see      java.util.zip;
*
*/

public class InterfaceLoader {

    /* Vectorul cu elementele vine de mai sus, deci trebuie transmis
     * in constructor
     */
    private Vector elements = new Vector();
    private File   fileToLoad;

    /**
    *   Constructor tipic, trebuie transmise elementele prin intermediul
    * unui vector, elementele fiind JavinstWrapper-e
    * @param elements   Vector cu elementele interfatei wrappate in
    *                   JavinstWrapper-e
    * @param fileToLoad Handlerul fisierului sub care se salveaza
    */
    public InterfaceLoader( File fileToLoad) {
        this.fileToLoad = fileToLoad;
    }

    /**
    *   Constructor, trebuie transmise elementele prin intermediul
    * unui vector, elementele fiind JavinstWrapper-e
    * @param elements   Vector cu elementele interfatei wrappate in
    *                   JavinstWrapper-e
    * @param fileToLoad Calea + numele fisierului sub care se salveaza
    */
    public InterfaceLoader(String fileToLoad) {
        try {
            this.fileToLoad = new File(fileToLoad);
        } catch (NullPointerException ex ){
            new Message (new Frame(), "Error", "No path speciffied !");
        }
    }

    /**
    *   Metoda de baza care ceaza arhiva, si serializeaza componentele
    *   Salvarea interfetei presupune urmatoarele etape:
    *       1. Crearea arhivei care va contine fisierele independente
    *          cu obiectele serializate
    *       2. Se creaza un zipEntry nou pentru fiecare obiect de serializat
    *       3. Se serializeaza componenta
    */

    public Vector loadInterface() {
        FileInputStream   fip;
        ObjectInputStream oip;
        JavinstWrapper    jwr;
            try{
                fip= new FileInputStream(fileToLoad);
                oip= new ObjectInputStream(fip);
                int i = (int)oip.readInt();
                for(int j = 0 ; j < i; j++){
                    jwr = (JavinstWrapper)oip.readObject();
                    elements.addElement(jwr);
                }
                oip.close();
                fip.close();
                return elements;
            }catch (Exception ex) {
                System.err.println("Problems at InterfaceLoader-> loadInterface()");
                ex.printStackTrace();
                new Message(new Frame(), "Error", "Can't load Interface !");
                return elements;
            }
    }
}
