/**********************************************************************************
 * ClassName  :   InterfaceSaver
 * Date       :   17, May  1998
 * Autor      :   Remus Pereni [RP]
 * Description:   Vezi mai jos
 **********************************************************************************/

package JaRPE;

import java.io.*;
import java.util.zip.*;
import java.util.Vector;
import java.awt.Frame;

/**
*   Clasa care asigura serializarea tuturor obiectelor in mod unitar
* intr-un fisier ZIP, necompresat. Idea este ca o interfata are mai
* multe elemente care serializate ar fi mai multe fisiere, in acest
* mod (prin zipare) desi fiecare obiect consista dintr-un fisier
* independent din exterior avem de lucru numai cu un singur fisier
* , arhiva care contine toate elementele serializate ale interfatei.
*
* @author   Remus Pereni
* @version  0.01, 17 May 1998 02:48 PM
* @see      java.util.zip;
*
*/

public class InterfaceSaver {

    /* Vectorul cu elementele vine de mai sus, deci trebuie transmis
     * in constructor
     */
    private Vector elements ;
    private File   fileToSave;

    /**
    *   Constructor tipic, trebuie transmise elementele prin intermediul
    * unui vector, elementele fiind JavinstWrapper-e
    * @param elements   Vector cu elementele interfatei wrappate in
    *                   JavinstWrapper-e
    * @param fileToSave Handlerul fisierului sub care se salveaza
    */
    public InterfaceSaver(Vector elements, File fileToSave) {
        this.elements   = elements;
        this.fileToSave = fileToSave;
    }

    /**
    *   Constructor, trebuie transmise elementele prin intermediul
    * unui vector, elementele fiind JavinstWrapper-e
    * @param elements   Vector cu elementele interfatei wrappate in
    *                   JavinstWrapper-e
    * @param fileToSave Calea + numele fisierului sub care se salveaza
    */
    public InterfaceSaver(Vector elements, String fileToSave) {
        this.elements   = elements;
        try {
            this.fileToSave = new File(fileToSave);
        } catch (NullPointerException ex ){
            new Message (new Frame(), "Error", "No path speciffied !");
        }
    }

    /**
    *   Metoda de baza care ceaza arhiva, si serializeaza componentele
    *   Salvarea interfetei presupune urmatoarele etape:
    *       1. Crearea arhivei care va contine fisierele independente
    *          cu obiectele serializate
    *       2. Se creaza un zipEntry nou pentru fiecare obiect de serializat
    *       3. Se serializeaza componenta
    */

    public void saveInterface() {
        FileOutputStream   fop;
        ZipOutputStream    zop;
        ObjectOutputStream oop;
            try{
                fop= new FileOutputStream(fileToSave);
                oop= new ObjectOutputStream(fop);
                //zop= new ZipOutputStream(oop);
                oop.writeInt(elements.size());
                for (int i = 0; i < elements.size(); i++) {
                    JavinstWrapper jwr = (JavinstWrapper)elements.elementAt(i);
                    //rzop.putNextEntry(new ZipEntry(i + ".ser"));
                    oop.writeObject(jwr);
                }
                oop.close();
               // zop.close();
                fop.close();
            }catch (Exception ex) {
                System.err.println("Problems at InterfaceSaver-> saveInterface()");
                ex.printStackTrace();
                new Message(new Frame(), "Error", "Can't save Interface !");
            }
    }
}
