package JaRPE.JaRPEdit;

import java.beans.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.Toolkit.*;
import java.io.*;
import java.util.Vector;
import JaRPE.*;


class ComponentBoxPanel extends Panel implements MouseListener, ActionListener {
    private JaRPEdit  parent;
    private WorkFrame actionFrame;
    private TextField display;
    private Panel p;
    private ComponentBoxPanel componentBoxPanel;

    ComponentBoxPanel(JaRPEdit parent, Frame frame) {
    	this.frame = frame;
    	this.parent= parent;
    	this.componentBoxPanel = this;
    	setLayout(new BorderLayout());
    	setBackground(SystemColor.control);
    	setFont(new Font("Dialog", Font.PLAIN, 10));
    	addMouseListener(this);
        p = new Panel(new FlowLayout(FlowLayout.LEFT));
    	Vector jarNames = getJarNames();
    	for (int i = 0; i < jarNames.size(); i++) {
    	    String name = (String)jarNames.elementAt(i);
    	    addBeansInJar(name);
    	}
  	    for (int i = 0; i < beanLabels.size(); i++) {
    	    String name = (String)beanLabels.elementAt(i);
    	    Image  img= (Image)beanIcons.elementAt(i);
            TrickyButton tp = new TrickyButton(name, 24, 24, img);
            tp.addActionListener(this);
            tp.addMouseListener(this);
            p.add(tp);
            validate();
        }

        add("Center", p);
        display = new TextField();
        display.setFont(new Font("Helvetica", Font.PLAIN, 12));
        add("South", display);
    }

    public void actionPerformed(ActionEvent e) {
        notifyAddComponent(e.getActionCommand());
    }

    public void refreshComponents() {
        p.removeAll();
  	    for (int i = 0; i < beanLabels.size(); i++) {
    	    String name = (String)beanLabels.elementAt(i);
    	    Image  img= (Image)beanIcons.elementAt(i);
            TrickyButton tp = new TrickyButton(name, 24, 24, img);
            tp.addActionListener(this);
            tp.addMouseListener(this);
            p.add(tp);
            validate();
        }
    }

    void notifyAddComponent(String name) {
        System.err.println("Se adauga: " + name);
        int idx = beanLabels.indexOf(name);
        JarInfo  ji = (JarInfo)beanJars.elementAt(idx);
        String beanName = (String) beanNames.elementAt(idx);
        try {
              bean = ji.getInstance(beanName);
              if (bean != null) System.out.println("Ok: " + beanName);
        } catch (Exception ex) {
            error("instantion of a new bean failed", ex);
    		return;
	    }
        (parent.getWorkFrame()).addComponent(bean, beanName, name);

    }

    // keep it with package scope for the inner class' sake
    void addWithUniqueName(Vector v, String s) {
	    if (! v.contains(s)) {
	        debug("adding "+s);
    	    v.addElement(s);
    	    return;
    	}
    	int count = 2;
    	while (v.contains(s+"<"+count+">")) {
    	    count += 1;
    	}
    	debug("adding "+s+"<"+count+">");
    	v.addElement(s+"<"+count+">");
    	return;
    }

    private class Helper implements DoOnBean {
	public void action(JarInfo ji, BeanInfo bi, String beanName) {
	    String label;
	    Image img = null;

	    BeanDescriptor bd = bi.getBeanDescriptor();
	    Class beanClass = bd.getBeanClass();
	    if (beanName.equals(beanClass.getName())) {
    		label = bi.getBeanDescriptor().getDisplayName();
    		img = bi.getIcon(BeanInfo.ICON_COLOR_16x16);
    		if (img ==null) img = ((frame.getToolkit()).getImage((System.getProperty("user.dir")
    		            + File.separatorChar + "images"
    		            + File.separatorChar + "JBIcon16.gif")));


	    }
	    else {
	        label = beanName;
		    int ix = beanName.lastIndexOf('.');
    		if (ix >= 0) {
    		    label = beanName.substring(ix+1);
       		}
    		img = ((frame.getToolkit()).getImage(System.getProperty("user.dir")
    		            + File.separatorChar + "images"
    		            + File.separatorChar + "JBIcon16.gif"));
  	    }
	    addWithUniqueName(beanLabels, label);
	    beanNames.addElement(beanName);
	    beanIcons.addElement(img);
	    beanJars.addElement(ji);

	}

	public void error(String msg) {
	    ComponentBoxPanel.this.error(msg);
	}
	public void error(String msg, Exception ex) {
	    ComponentBoxPanel.this.error(msg, ex);
	}
    }

    private Helper helper = new Helper();

    /**
     * Add all the beans in a Jar file into the ToolBox
     */
    synchronized void addBeansInJar(String jarFile) {
    	JarLoader.loadJarDoOnBean(jarFile, helper);
    }

    /**
     * Special action.
     *
     * Currently only used to register the BeanBox
     */
    synchronized void addBeanClass(String beanClassName) {
    	Class beanClass;
    	try {
    	    beanClass = Class.forName(beanClassName);
    	} catch (Exception ex) {
    	    System.err.println("Componentbox: couldn't instantiate " +  beanClassName);
    	    return;
   	    }

    	BeanInfo bi;
    	try {
    	    bi = Introspector.getBeanInfo(beanClass);
    	} catch (Exception ex) {
    	    System.err.println("Toolbox: couldn't find BeanInfo for " + beanClassName);
    	    ex.printStackTrace();
    	    return;
    	}
    	String label = bi.getBeanDescriptor().getDisplayName();
    	Image img = bi.getIcon(BeanInfo.ICON_COLOR_16x16);
        if (img ==null) img = ((frame.getToolkit()).getImage(System.getProperty("user.dir")
    		            + File.separatorChar + "images"
    		            + File.separatorChar + "JBIcon16.gif"));
	    addWithUniqueName(beanLabels, label);
    	beanNames.addElement(beanClassName);
    	beanIcons.addElement(img);
    	beanJars.addElement(null);
    }

    /**
     * Provide information on all the loaded JAR files, regardless of origin.
     */
    public Vector getLoadedJarInfo() {
	return beanJars;
    }

    private static Vector getJarNames() {
	File cwd = new File(System.getProperty("user.dir"));
	File pwd = new File(cwd.getParent());
	File jars = new File(pwd, "jars");

	if (! jars.isDirectory()) {
	    System.err.println(jars+" is not a directory!!");
	}

	Vector result = new Vector();
	String names[];
	names = jars.list(new FileExtension(".jar"));
	for (int i=0; i<names.length; i++) {
	    result.addElement(jars.getPath() + File.separatorChar + names[i]);
	}
	names = jars.list(new FileExtension(".JAR"));
	for (int i=0; i<names.length; i++) {
	    result.addElement(jars.getPath() + File.separatorChar + names[i]);
	}

	return result;
    }


    //----------------------------------------------------------------------

    // Mouse listener methods for this panel.

    public void mouseClicked(MouseEvent evt) {
    }

    public void mousePressed(MouseEvent evt) {
    }

    public void mouseReleased(MouseEvent evt) {
    }

    public void mouseEntered(MouseEvent evt) {
        if (evt.getComponent() instanceof TrickyButton){
            display.setText(((TrickyButton)evt.getComponent()).getActionCommand());
        }
    }

    public void mouseExited(MouseEvent evt) {
    }

    public void mouseDragged(MouseEvent evt) {
    }

    public void mouseMoved(MouseEvent evt) {
    }

    //----------------------------------------------------------------------

    public void error(String message, Exception ex) {
	String mess = message + ":\n" + ex;
	System.err.println(message);
	ex.printStackTrace();
	// Popup an ErrorDialog with the given error message.
	new ErrorDialog(frame, mess);

    }

    public void error(String message) {
	String mess = message;
	System.err.println(message);
	// Popup an ErrorDialog with the given error message.
	new ErrorDialog(frame, mess);

    }

    static  void debug(String msg) {
	if (debug) {
	    System.err.println("ToolBox:: "+msg);
	}
    }

    Vector beanLabels = new Vector();
    Vector beanNames = new Vector();
    Vector beanIcons = new Vector();
    Vector beanJars = new Vector();
    private int topPad = 0;
    private int sidePad = 0;
    private final static int rowHeight = 20;
    private Thread insertThread = null;
    private static boolean debug = false;
    private Object pendingBean;
    private String pendingBeanLabel;
    private Frame frame;
    private Object bean;

    private static Cursor crosshairCursor = Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR);
    private static Cursor defaultCursor = Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR);
}


// Auxiliary class

class FileExtension implements FilenameFilter {
    private String extension;

    public FileExtension(String ext) {
	this.extension = ext;
    }

    public boolean accept(File dir, String name) {
	return name.endsWith(extension);
    }
}
