package JaRPE.JaRPEdit;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.beans.*;
import java.applet.*;
import java.io.*;
import JaRPE.*;

public class WorkFrame extends Frame implements MouseListener, MouseMotionListener, Runnable {

    private Frame   parent;
    private Object  bean ;
    private String  beanName, beanLabel;
    private boolean insert;
    private boolean active = false;
    private int     pasteX, pasteY;
    private static SmartPanelProp smp;
    private Wrapper smpWrapper;

    transient Component connectionSource;
    transient int oldX1 = -1;
    transient int oldX2;
    transient int oldY1;
    transient int oldY2;
    transient Rectangle oldRubberBox;

    transient Hashtable methodMap;   // maps MenuItems to Methods
    transient Hashtable eventMap;   // maps Menus to EventSetDescriptors
    transient Class customizerClass;

    private static Hashtable serNames = new Hashtable();

    // If moveChild is non-null, we're moving that child
    transient Wrapper moveChild;
    transient int moveStartX;
    transient int moveStartY;

    // If resizeChild is non-null, we're resizing that child
    transient Wrapper resizeChild;
    transient Wrapper currentBean;
    transient Cursor resizeCursor;
    transient int resizeStartX;
    transient int resizeStartY;

    private final static int padX = 10;
    private final static int padY = 10;

    private final static String defaultStoreFile = "beanbox.tmp";
    private final static String defaultSerializeFile = "example.ser";
    private static String clipDir = "tmp";
    private static String clipFile = "JaRPEClip.ser";
    private static String clipResource = "JaRPEClip";
    private static String tmpDir = "tmp";
    private static String clipLabel;


    // We keep a private event queue and a thread to process them.
    transient private Thread eventThread;
    transient private Vector events;


    // MouseClickEvent describes the moset recent mouse click event.
    private transient MouseEvent mouseClickEvent;
    private transient long mousePressedWhen = 0;

    // Shorthands for the cursors.
    private static Cursor nwResizeCursor = Cursor.getPredefinedCursor(Cursor.NW_RESIZE_CURSOR);
    private static Cursor neResizeCursor = Cursor.getPredefinedCursor(Cursor.NE_RESIZE_CURSOR);
    private static Cursor swResizeCursor = Cursor.getPredefinedCursor(Cursor.SW_RESIZE_CURSOR);
    private static Cursor seResizeCursor = Cursor.getPredefinedCursor(Cursor.SE_RESIZE_CURSOR);
    private static Cursor crosshairCursor = Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR);
    private static Cursor defaultCursor = Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR);

    //
    WorkFramePopupMenu wfpopm;

    public WorkFrame (Frame parent,int x, int y) {
        super("New Remote Interface");
        this.parent = parent;
        addMouseListener(this);
        addMouseMotionListener(this);
        setLayout(null);
      	setIconImage(getToolkit().getImage("images/edit32.gif"));
    	setLocation(x, y);
        add(wfpopm = new WorkFramePopupMenu());
        setSize(500, 400);
        setVisible(true);
        smp = new SmartPanelProp((Container)this);
        setBackground(smp.getInterfaceBackground());
        setForeground(smp.getInterfaceForeground());
        setFont(smp.getInterfaceFont());
        setTitle(smp.getInterfaceTitle());
        repaint();
    }

    public WorkFrame (Frame parent,int x, int y, SmartProp sp) {
        super();
        this.parent = parent;
        addMouseListener(this);
        addMouseMotionListener(this);
        setLayout(null);
      	setIconImage(getToolkit().getImage("images/edit32.gif"));
    	setLocation(x, y);
        add(wfpopm = new WorkFramePopupMenu());
        setSize(500, 400);
        setVisible(true);
        smp = new SmartPanelProp((Container)this, sp);
        setBackground(smp.getInterfaceBackground());
        setForeground(smp.getInterfaceForeground());
        setFont(smp.getInterfaceFont());
        setTitle(smp.getInterfaceTitle());
        repaint();
    }


    public synchronized void dispose() {
        ((JaRPEdit)parent).setWorkFrame(null);
        super.dispose();
    }

    public void addComponent(Object o, String beanName, String beanLabel) {
        this.insert   = true;
        this.bean     = o;
        this.beanName = beanName;
        this.beanLabel= beanLabel;
        setCursor(Cursor.CROSSHAIR_CURSOR);
    }

    public void removeComponent(Component comp){
        remove(comp);
        paint(getGraphics());
    }

    public void doInsert(Object bean, String beanName, String beanLabel, int xcoord, int ycoord) {
        Wrapper wr = new Wrapper(bean, beanName, beanLabel);
        add(wr);
        wr.setLocation(xcoord, ycoord);
    	int childWidth = wr.getPreferredSize().width;
	    int childHeight= wr.getPreferredSize().height;
    	 //int x = xcoord - childWidth/2;
    	 //int y = ycoord - childHeight/2;
    	 int x = xcoord;
    	 int y = ycoord;
    	wr.setBounds(x, y, childWidth, childHeight);
    	if (Beans.isInstanceOf(bean, Applet.class)) {
    	    Applet apl = (Applet) Beans.getInstanceOf(bean, Applet.class);
    	    apl.start();
    	}
        paint(getGraphics());
        setCursor(Cursor.DEFAULT_CURSOR);
        setCurrentBean(wr);
    }

    public void setCurrentBean(Wrapper wr) {
      if (!(wr.getBean() instanceof SmartPanelProp)) {
            Wrapper oldbean = currentBean;
            this.currentBean = wr;
            if (oldbean != null) oldbean.repaint();
            wr.repaint();
      }
    }

    public Wrapper getCurrentBean() {
        return currentBean;
    }

    public SmartPanelProp getSmartPanelProp() {
        return smp;
    }

    public void setSmartPanelProp(SmartPanelProp smp) {
        this.smp = smp;
    }

    public Wrapper getSmartPanelWrapper() {
        if (smpWrapper == null) {
                smpWrapper = new Wrapper(getSmartPanelProp(),
                                         "JaRPE.SmartPanelProp", "SmartPanelProp");
        }
    return smpWrapper;
    }


    protected void paste() {
    	synchronized (this) {
    	    mouseClickEvent = null;
        }
    	try {
            // Set the insert cursor before reading the clipboard.
    	    setCursor(crosshairCursor);

    	    SimpleClassLoader loader = SimpleClassLoader.ourLoader;
    	    MyProducer p = new MyProducer(getClipFileName());
    	    String clipName = getClipResource();

    	    loader.setLocalResourceSource(clipName+".ser", p);
    	    bean = Beans.instantiate(loader, clipName);
    	    beanLabel = getClipLabel();
    	    beanName  = (bean.getClass()).getName();
    	    insert = true;
    	    //doInsert(bean, beanName, beanLabel, pasteX, pasteY);
    	} catch (Exception ex) {
    	    new Message(this, "Atention", "Paste failed");
       	}
    }

    public void setActive(boolean fan) {

        this.active = fan;
        Beans.setDesignTime(!fan);
        Component[] comp = getComponents();
        for (int i=0 ; i < getComponentCount() ; i ++){
            if ( comp[i] instanceof Wrapper )
                    ((Wrapper)comp[i]).setActive(fan);
        }
    }

    Rectangle getResizeBox(int mx, int my) {
    	int x = resizeChild.getLocation().x;
    	int y = resizeChild.getLocation().y;
    	int w = resizeChild.getSize().width;
    	int h = resizeChild.getSize().height;
    	if (resizeCursor == nwResizeCursor) {
    	    w = w - mx;
    	    h = h - my;
	        x = x + mx;
    	    y = y + my;
        } else if (resizeCursor == swResizeCursor) {
    	    w = w - mx;
    	    h = my;
    	    x = x + mx;
        } else if (resizeCursor == neResizeCursor) {
    	    w = mx;
    	    h = h - my;
    	    y = y + my;
        } else if (resizeCursor == seResizeCursor) {
    	    w = mx;
    	    h = my;
    	}
    	if (w < 10) {
    	    w = 10;
    	}
    	if (h < 10) {
    	    h = 10;
    	}
    	return new Rectangle(x, y, w, h);
    }

    /**
     * Someone has done a mouse button down on a corner of the "wrapper"
     * around the bean.  This starts a resize sequence that will be
     * terminated when they release the mouse button.  We will draw a
     * rubbed-banded box to indicate the proposed resize shape..
     */
    void startResize(Wrapper child, int x, int y, Cursor cursor) {
    	resizeStartX = child.getLocation().x + x;
    	resizeStartY = child.getLocation().y + y;
    	resizeChild = child;
    	resizeCursor = cursor;
    }

    void finishResize(int mx, int my) {
    	Rectangle box = getResizeBox(mx, my);
    	resizeChild.setBounds(box.x, box.y, box.width, box.height);
    	resizeChild.doLayout();
    	deleteRubberBox();
    	resizeChild = null;
    }


    /**
     * Someone has done a mouse button down on the "wrapper" around the bean.
     * This starts a move that will be terminated when they release the
     * mouse button.  We will draw a rubbed-banded box to indicate the
     * current move position.
     */
    void startMove(Wrapper child, int x, int y) {
    	moveChild = child;
    	moveStartX = x;
    	moveStartY = y;
    }
    /**
     * Figure out the on-screen box to be drawn to represent the proposed
     * new move location of a bean we're moving.
     */
    Rectangle getMoveBox(int mx, int my) {
    	int x = moveChild.getLocation().x;
    	int y = moveChild.getLocation().y;
    	int w = moveChild.getSize().width;
    	int h = moveChild.getSize().height;
    	x = x + mx - moveStartX;
    	y = y + my - moveStartY;
    	return new Rectangle(x, y, w, h);
     }
    /**
     * Complete a move started with "startMove".
     */
    void finishMove(int mx, int my) {
    	Rectangle box = getMoveBox(mx, my);
    	moveChild.setBounds(box.x, box.y, box.width, box.height);
    	deleteRubberBox();
    	moveChild = null;
    }

    private void drawRubberBox(Rectangle box) {
    	Graphics g = getGraphics();
    	if (g == null) {
    	    return;
    	}
    	// First remove any existing rubber box.
    	deleteRubberBox();
    	// Now draw the new rubber box.
    	g.setColor(Color.red);
    	g.drawRect(box.x, box.y, box.width, box.height);
    	oldRubberBox = box;
    }

    private void deleteRubberBox() {
    	if (oldRubberBox != null) {
    	    Graphics g = getGraphics();
    	    if (g == null) {
    	        return;
    	    }
    	    g.setColor(getBackground());
    	    g.drawRect(oldRubberBox.x, oldRubberBox.y,
    		oldRubberBox.width, oldRubberBox.height);
    	    oldRubberBox = null;
    	}
    }

    public void run() {

    }

    public static String getClipDir() {
    	return clipDir;
    }

    public static String getClipFileName() {
    	if (clipDir != null) {
    	    return clipDir+java.io.File.separator+clipFile;
    	} else {
    	    return clipFile;
    	}
    }

    public static String getClipResource() {
    	return clipResource;
    }

    public static String getTmpDir() {
    	return tmpDir;
    }

    public static void setClipLabel(String label) {
        clipLabel = label;
    }

    public static String getClipLabel() {
        return clipLabel;
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mouseClicked(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
    }

    public synchronized void mouseMoved(MouseEvent evt) {

    }

    public synchronized void mouseDragged(MouseEvent evt) {
	int x = evt.getX();
	int y = evt.getY();

	// If we're in the middle of a move, draw a rubber banded box.
	if (moveChild != null) {
	    // Make sure the event coordinates are relative to the child.
	    if (evt.getComponent() == this) {
		x -= moveChild.getLocation().x;
		y -= moveChild.getLocation().y;
	    }
	    drawRubberBox(getMoveBox(x, y));
	}

        // If we're in the middle of a resize, draw a rubber-banded
	// resize box.
	if (resizeChild != null) {
	    // Make sure the event coordinates are relative to the child.
	    if (evt.getComponent() == this) {
		x -= resizeChild.getLocation().x;
		y -= resizeChild.getLocation().y;
	    }
	    drawRubberBox(getResizeBox(x, y));
        }
    }

    public void mouseReleased(MouseEvent e) {
        long deltaWhen = (e.getWhen() - mousePressedWhen);
    	mousePressedWhen = 0;

    	int x = e.getX();
    	int y = e.getY();

    	// If we're in the middle of a move, complete the move.
    	if (moveChild != null) {
    	    // Make sure the event coordinates are relative to the child.
    	    if (e.getComponent() == this) {
        		x -= moveChild.getLocation().x;
        		y -= moveChild.getLocation().y;
      	    }
    	    // OK, insert the current object at the mouse.
            deleteRubberBox();
    	    finishMove(x, y);
    	    return;
    	}

        // If we're in the middle of a resize, complete the resize.
    	if (resizeChild != null) {
    	    // Make sure the event coordinates are relative to the child.
    	    if (e.getComponent() == this) {
    	    	x -= resizeChild.getLocation().x;
        		y -= resizeChild.getLocation().y;
            }
	        deleteRubberBox();
     	    finishResize(x, y);
	        return;
        }

    	// Because of bug #4039858 we get a bogus mouseClicked and mouseReleased
    	// event when a menuItem is selected.  Therefore we manually check
    	// if there has been a mousePressed within the preceeding 5 seconds before
    	// the mouseRelease, and only then do we treat it as a genuine mouse
    	// click.						KGH 4/3/97
    	if (deltaWhen < 5000) {
    	    // OK, treat this as a genuine mouse click.
    	    // Wakeup anyway who is waiting for a mouse down.
    	    synchronized (this) {

            	mouseClickEvent = new MouseEvent(e.getComponent(), e.getID(),
	        	        e.getWhen(), e.getModifiers(), e.getX(), e.getY(),
        	 	        e.getClickCount(), e.isPopupTrigger());
    	        notifyAll();

    	        // set the new focus, unless we're in the middle of an insert.
    	        if (getCursor() != crosshairCursor) {
        	            Component target = e.getComponent();
        //    	            BeanBoxFrame.setCurrentComponent(target);
    	    	}
    	    }
	    }



     if (!(e.getSource() instanceof WorkFrame)) return;
     if (e.isPopupTrigger() && !active) {
        wfpopm.show(this, e.getX(), e.getY());
        pasteX= e.getX();
        pasteY= e.getY();
   	    return;
     }
     if (insert == true) {
        Wrapper wr = new Wrapper(bean, beanName, beanLabel);
        add(wr);
        setCurrentBean(wr);
        wr.setLocation(e.getX(), e.getY());
    	int childWidth = wr.getPreferredSize().width;
	    int childHeight= wr.getPreferredSize().height;
    	 x = e.getX() - childWidth/2;
    	 y = e.getY() - childHeight/2;
    	wr.setBounds(x, y, childWidth, childHeight);
    	if (Beans.isInstanceOf(bean, Applet.class)) {
    	    Applet apl = (Applet) Beans.getInstanceOf(bean, Applet.class);
    	    apl.start();
    	}
        paint(getGraphics());
        setCursor(Cursor.DEFAULT_CURSOR);
        insert = false;
        	// Make the child the focus.
      //  parent.setCurrentComponent(child);
     }

    }


}

class WorkFramePopupMenu extends PopupMenu implements ActionListener {
    private String beanName;
    private String beanLabel;


  	public WorkFramePopupMenu () {
        super();
    	add(new MenuItem("Properties"));
    	addSeparator();
        add(new MenuItem("Paste"));
        addActionListener(this);
    }

    public void actionPerformed(ActionEvent ev) {
        /*S-a selectat Properties in PopupMenu asociat componentei
         * in acest caz PropertiesFrame face restul de munca
         */
        if (ev.getActionCommand() == "Properties"){
            Component pa;
            pa =(Component)getParent();
            while(!(pa instanceof Frame)){ pa = (Component)pa.getParent();}
            PropertySheet pf = new PropertySheet( ((WorkFrame)pa).getSmartPanelWrapper()
                                                 , (pa.getLocation()).x + 100
                                                 , (pa.getLocation()).y + 100);
            new WindowCloser(pf, false);
        }
        if (ev.getActionCommand() == "Paste"){
            Component pa;
            pa = (Component)getParent();
            while (!(pa instanceof WorkFrame)) pa= (Component)pa.getParent();
            ((WorkFrame)pa).paste();

        }
    }
}


/**
 * Auxiliary class to deliver the InputStream
 */
class MyProducer implements InputStreamProducer{
    private String name;

    public MyProducer(String fileName) {
    	this.name = fileName;
    }

    public synchronized InputStream getInputStream() {
    	try {
    	    return new FileInputStream(name);
    	} catch (Exception ex) {
    	    return null;
    	}
    }
}

