package JaRPE.JaRPEdit;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.beans.*;
import java.util.*;
import java.lang.reflect.Method;
import java.applet.Applet;
import JaRPE.*;

public class Wrapper extends Panel implements Serializable, MouseListener, MouseMotionListener {

    //----------------------------------------------------------------------

    private Component child;
    private Object bean;
    private String beanName, beanLabel;
    private WrapperPopupMenu popmen;
    private transient boolean active;
    private transient Cursor cursor = defaultCursor;

    private final static int borderWidth = 5;
    private final static int resizeDelta = 8;

    private static Cursor nwResizeCursor = Cursor.getPredefinedCursor(Cursor.NW_RESIZE_CURSOR);
    private static Cursor neResizeCursor = Cursor.getPredefinedCursor(Cursor.NE_RESIZE_CURSOR);
    private static Cursor swResizeCursor = Cursor.getPredefinedCursor(Cursor.SW_RESIZE_CURSOR);
    private static Cursor seResizeCursor = Cursor.getPredefinedCursor(Cursor.SE_RESIZE_CURSOR);
    private static Cursor moveCursor     = Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR);
    private static Cursor defaultCursor  = Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR);
                   Cursor newCursor = defaultCursor;

    private static Vector invisibleWrappers = new Vector();
    private transient boolean sawMouseDown;

    private transient Hashtable esdMap;
    private Vector eventTargets;

    private static int hashBarWidth = 4;
    private static int hashBarLength = 104;
    private static Image xHashBar;
    private static Image yHashBar;
    private Point oldlocation;

    //----------------------------------------------------------------------


    public Wrapper(Object bean, String beanName, String beanLabel) {
        System.out.println(beanName + " : " + beanLabel);
        this.bean      = bean;
        this.beanLabel = beanLabel;
    	if (beanName == null) {
    	    beanName = bean.getClass().getName();
    	}
    	this.beanName = beanName;
    	setLayout(null);
    	if (Beans.isInstanceOf(bean, Component.class)) {
                child = (Component)Beans.getInstanceOf(bean, Component.class);
    	}
    	popmen= new WrapperPopupMenu(beanName, beanLabel);
    	this.add(child);
    	this.add(popmen);
    	this.addMouseListener(this);
    	this.addMouseMotionListener(this);
    	child.setLocation(borderWidth, borderWidth);
        initialize();
    	doLayout();
    }

    void initialize() {
    	esdMap = new Hashtable();
    	try {
    	    BeanInfo bi = Introspector.getBeanInfo(bean.getClass());
    	    EventSetDescriptor[] esds = bi.getEventSetDescriptors();
    	    for (int i = 0; i < esds.length; i++) {
    	        esdMap.put(esds[i].getName(), esds[i]);
            }
	    } catch (IntrospectionException ex) {
    	    System.err.println("Wrapper couldn't introspect on bean: " + bean);
        }
	    if (eventTargets == null) {
    	    eventTargets = new Vector();
    	}
    }


    private void readObject(ObjectInputStream s) throws ClassNotFoundException, IOException {
    	s.defaultReadObject();
    	initialize();
    }


    void writeNakedBean(ObjectOutputStream oos) throws IOException {
    	oos.writeObject(bean);
    }
    void cleanup() {
    	if (bean instanceof Applet) {
    	    Applet apl = (Applet)bean;
    	    apl.stop();
    	    apl.destroy();
    	}
    }

    public Object getBean() {
    	return bean;
    }

    public void prepareSaveBean() {
        oldlocation = getChild().getLocation();
        getChild().setLocation(getLocation().x+5, getLocation().y+5);
    }

    public void restoreSaveBean() {
    	getChild().setLocation(oldlocation.x, oldlocation.y);

    }
    public String getBeanName() {
    	return beanName;
    }

    public String getBeanLabel() {
    	return beanLabel;
    }

    public Component getChild() {
    	return child;
    }

    public void doLayout() {
    	child.doLayout();
    	sizeToFit();
    	child.repaint();
    }

    private void sizeToFit() {
    	if (childHasStupidPreferredSize()) {
    	    return;
    	}
    	Dimension d = getPreferredSize();
    	setSize(d.width, d.height);
    }

    public void setActive(boolean isActive) {
    	active = isActive;
    	repaint();
    }

    public Dimension getPreferredSize() {
    	Dimension childSize = child.getPreferredSize();
    	if (childHasStupidPreferredSize()) {
    	    childSize = child.getSize();
    	}
    	int width = childSize.width;
    	int height = childSize.height;
    	Dimension minSize = child.getMinimumSize();
    	if (minSize.height > height) {
    	    height = minSize.height;
    	}
    	if (minSize.width > width) {
	        width = minSize.width;
    	}
    	width += (2 * borderWidth);
    	height += (2 * borderWidth);
    	return new Dimension(width, height);
    }

    private boolean childHasStupidPreferredSize() {
    	if (child instanceof Panel) {
    	    Container cont = (Container)child;
    	    LayoutManager lay = cont.getLayout();
    	    if (cont.getComponentCount() == 0 && lay instanceof FlowLayout) {
        		FlowLayout flow = (FlowLayout) lay;
        		Dimension cd = child.getPreferredSize();
        		Dimension fd = flow.preferredLayoutSize(cont);
        		if (cd.width == fd.width && cd.height == fd.height) {
            		    return true;
        		}
    	    }
    	}
    	return false;
    }

    public void setBounds(int x, int y, int width, int height) {
    	super.setBounds(x, y, width, height);
    	child.setBounds(borderWidth, borderWidth, width-(2*borderWidth), height-(2*borderWidth));
    	child.validate();
    }

    public void paint(Graphics g) {
    	if (!active && Beans.isDesignTime()) {
    	    int width = getSize().width;
    	    int height = getSize().height;

    	    if (((WorkFrame)getParent()).getCurrentBean() == this) {
        	    getHashBars(this);

        	    int nudge = 2 * hashBarWidth;
        	    int bottomNudge = - ((height-hashBarWidth) % nudge);
        	    for (int x = 0; x < width; x += hashBarLength) {
        	        g.drawImage(xHashBar, x, 0, null);
        	        g.drawImage(xHashBar, x + bottomNudge, height-hashBarWidth, null);
        	    }
        	    int rightNudge = - ((width-hashBarWidth) % nudge);
        	    for (int y = 0; y < height; y += hashBarLength) {
        	        g.drawImage(yHashBar, 0, y, null);
        	        g.drawImage(yHashBar, width-hashBarWidth, y+rightNudge, null);
        	    }
            }
    	}
    	super.paint(g);
    }

    public synchronized void activate() {
    	active = true;
    	repaint();
    }

    public synchronized void deactivate() {
    	active = false;
    	repaint();
    }

   public synchronized void doMouseStuff(MouseEvent evt) {
    	if (active || !Beans.isDesignTime()) {
    	    return;
    	}
    	Component parent = child.getParent();
     	if (parent != null && parent instanceof WorkFrame) {
    	    return;
    	}

    	int x = evt.getX();
    	int y = evt.getY();
      	int id = evt.getID();

    	int lowX = borderWidth;
    	int highX = getSize().width - borderWidth;
    	int lowY = borderWidth;
    	int highY = getSize().height - borderWidth;

    	if (evt.isPopupTrigger()) {
    	    popmen.show(this, x, y);
    	    return;
    	}

    	boolean onTheBorder = ((evt.getSource() == this) &&
			(x < lowX || x > highX || y < lowY || y > highY));

    	if (onTheBorder && !active ) {
    	    if (id == MouseEvent.MOUSE_PRESSED) {
    	        if (cursor == nwResizeCursor ||
        		    cursor == swResizeCursor ||
        		    cursor == neResizeCursor ||
        		    cursor == seResizeCursor) {
    	        	    getWorkFrame().startResize(this, x, y, cursor);
	            } else if (cursor == moveCursor) {
        	    	    getWorkFrame().startMove(this, x, y);
    	        }
    	     } else if ((id == MouseEvent.MOUSE_ENTERED
		             || id == MouseEvent.MOUSE_MOVED) && ! active) {

        		lowX += resizeDelta;
        		lowY += resizeDelta;
        		highX -= resizeDelta;
        		highY -= resizeDelta;

    	        if (x < lowX && y < lowY) {
        		    newCursor = nwResizeCursor;
                } else if (x < lowX && y > highY) {
    	    	    newCursor = swResizeCursor;
    	        } else if (x > highX && y < lowY) {
    		        newCursor = neResizeCursor;
    	        } else if (x > highX && y > highY) {
    		        newCursor = seResizeCursor;
        		} else {
	        	    newCursor = moveCursor;
		     }
	    }
	    else newCursor =  defaultCursor;
	}

	if (newCursor != cursor) {
	    cursor = newCursor;
  	    if (!isJDK("1.1_Final")) {
		setCursor(cursor);
	    }
	}
    }

    final WorkFrame getWorkFrame() {
          return (WorkFrame)getParent();
    }

    protected void copy() {
    	getWorkFrame().setClipLabel(null);
	    try {
    	    File f = new File(getWorkFrame().getClipFileName());
    	    File dir = new File(f.getParent());
    	    dir.mkdirs();
            FileOutputStream fos = new FileOutputStream(f);
            ObjectOutputStream oos = new ObjectOutputStream(fos);
	        writeNakedBean(oos);
	        oos.close();
	        fos.close();
            getWorkFrame().setClipLabel(getBeanName());
    	} catch (Exception ex) {
	        new Message(getWorkFrame(), "Atention", "Copy failed");
       	}

    }

    protected void cut() {
    	Object bean = getBean();
    	    copy();
    	    Container parent = getParent();
    	    if (parent != null) {
    	    	parent.remove(this);
    	    }
    	    cleanup();
    }

    public void mouseClicked(MouseEvent evt) {
    	doMouseStuff(evt);
	    getWorkFrame().mouseClicked(evt);
    }

    public void mousePressed(MouseEvent evt) {
    	WorkFrame wf = (WorkFrame)getParent();
        wf.setCurrentBean(this);
        doMouseStuff(evt);
    	getWorkFrame().mousePressed(evt);
    }

    public void mouseReleased(MouseEvent evt) {
    	doMouseStuff(evt);
    	getWorkFrame().mouseReleased(evt);
    }

    public void mouseEntered(MouseEvent evt) {
    	doMouseStuff(evt);
    	getWorkFrame().mouseEntered(evt);
    }

    public void mouseExited(MouseEvent evt) {
    	doMouseStuff(evt);
    	getWorkFrame().mouseExited(evt);
    }

    public void mouseDragged(MouseEvent evt) {
    	doMouseStuff(evt);
	    getWorkFrame().mouseDragged(evt);
    }

    public void mouseMoved(MouseEvent evt) {
    	doMouseStuff(evt);
	    getWorkFrame().mouseMoved(evt);
    }

    private static synchronized void getHashBars(Component c) {
    	if (xHashBar != null) {
    	    return;
    	}
    	int len = hashBarLength + 20;

    	xHashBar = c.createImage(len, hashBarWidth);
    	yHashBar = c.createImage(hashBarWidth, len);

    	Polygon poly = new Polygon();
    	Graphics g = xHashBar.getGraphics();
    	for (int i = 0; i < 4; i++) {
    	    poly.addPoint(0,0);
    	}
       	poly.ypoints[2] = hashBarWidth;
       	poly.ypoints[3] = hashBarWidth;

    	for (int x = 0; x < (hashBarWidth+len); x += hashBarWidth) {
    	    if (((x/hashBarWidth)%2) == 0) {
    	        g.setColor(Color.darkGray);
    	    } else {
    	        g.setColor(Color.lightGray);
            }
            poly.xpoints[0] = x;
    	    poly.xpoints[1] = x + hashBarWidth;
    	    poly.xpoints[2] = x;
    	    poly.xpoints[3] = x - hashBarWidth;
    	    g.fillPolygon(poly);
    	}

    	g = yHashBar.getGraphics();
       	poly.xpoints[0] = 0;
       	poly.xpoints[1] = hashBarWidth;
       	poly.xpoints[2] = hashBarWidth;
       	poly.xpoints[3] = 0;

    	for (int y = 0; y < (hashBarWidth+len); y += hashBarWidth) {
    	    if (((y/hashBarWidth)%2) == 0) {
    	        g.setColor(Color.darkGray);
    	    } else {
    	        g.setColor(Color.lightGray);
    	    }
    	    poly.ypoints[0] = y;
    	    poly.ypoints[1] = y - hashBarWidth;
    	    poly.ypoints[2] = y;
    	    poly.ypoints[3] = y + hashBarWidth;
    	    g.fillPolygon(poly);
    	}
    }


    //----------------------------------------------------------------------
    private static Hashtable eventModelCache = new Hashtable();



    private static boolean isJDK(String version) {
    	return System.getProperty("java.version").equals(version);
    }
}


class WrapperEventTarget implements Serializable {
    static final long serialVersionUID = 4831901854891942741L;

    String eventSetName;
    Object targetBean;
    Object targetListener;
}




class WrapperPopupMenu extends PopupMenu implements ActionListener {
    private String beanName;
    private String beanLabel;


  	public WrapperPopupMenu (String beanName, String beanLabel) {
        super(beanName);
        this.beanName = beanName;
        this.beanLabel= beanLabel;
    	add(new MenuItem("Properties"));
    	addSeparator();
    	add(new MenuItem("Copy"));
        add(new MenuItem("Cut"));
        add(new MenuItem("Delete"));
        addActionListener(this);
    }

    public void actionPerformed(ActionEvent ev) {
        /*S-a selectat Properties in PopupMenu asociat componentei
         * in acest caz PropertiesFrame face restul de munca
         */
        if (ev.getActionCommand() == "Properties"){
            Component pa;
            pa =(Component)getParent();
            while(!(pa instanceof Frame)){ pa = (Component)pa.getParent();}
            Component wrapperparent = (Component)getParent();
            while(!(wrapperparent instanceof Wrapper)){ wrapperparent = (Component)wrapperparent.getParent(); }
            PropertySheet pf = new PropertySheet((Wrapper)wrapperparent, (pa.getLocation()).x + 100
                                                        , (pa.getLocation()).y + 100);
            new WindowCloser(pf, false);
        }
        /*S-a selectat staergere in PopupMenu asociat componentei
         * in acest caz ne autostergem face restul de munca
         */
        if (ev.getActionCommand() == "Delete"){
            Component frameparent, wrapperparent;
            frameparent = wrapperparent = (Component)getParent();
            while(!(frameparent instanceof Frame)){ frameparent = (Component)frameparent.getParent(); }
            while(!(wrapperparent instanceof Wrapper)){ wrapperparent = (Component)wrapperparent.getParent(); }
            ((WorkFrame)frameparent).removeComponent(wrapperparent);
        }

        /*S-a selectat copierea unei componente
         */
        if (ev.getActionCommand() == "Copy") {
            Component wrapperparent;
            wrapperparent = (Component)getParent();
            while(!(wrapperparent instanceof Wrapper)){ wrapperparent = (Component)wrapperparent.getParent(); }
            ((Wrapper)wrapperparent).copy();
        }

        /*S-a selectat copierea cu stergere a unei componente
         */
        if (ev.getActionCommand() == "Cut") {
            Component wrapperparent;
            wrapperparent = (Component)getParent();
            while(!(wrapperparent instanceof Wrapper)){ wrapperparent = (Component)wrapperparent.getParent(); }
            ((Wrapper)wrapperparent).cut();
        }

    }
}


