/**********************************************************************************
 * ClassName  :   BaseJavins
 * Date       :   17, Mai  1998
 * Autor      :   Remus Pereni [RP]
 * Description:   Vezi mai jos
 **********************************************************************************/


package JaRPE.Javins;


import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.*;
import java.net.*;
import java.lang.Thread;
import JaRPE.*;

/**
* BaseJavins - asa cum arata si numele se vrea a fi clasa de baza din care
* urmeaza a fi extinse totate Javins-urile.
* Este un bean clasic, fara a avea proprietatile de evenimente tipice beanurilo
* deoarece, nu erau justificate in cazul acetor componente (i-si inglobeaza toata
* functionalitatea lor)
*<p>
* Conceputa a fi o componenta generala atit de parte remote a procesului dar si in
* cadrul controlerelor de proces, datorita posibilitatilor de a prelua semnalele din
* resurse locale, pecum si de a efectua operatii de prelucarare (la dorinta) a
* semnalelor prin intermediul add-on urilor de tip <i> Modifier </i>
*
*<p>  Teoretic clasa respecta specificatiile de implementare a Beanurilor
*
* @author   Remus Pereni
* @version  0.01, 17 May 1998 01:30 AM
* @see      JaRPE.BaseJavins.RemoteSignalSource.html;
* @see      JaRPE.BaseJavins.LocalSignalSource.html;
* @see      JaRPE.BaseJavins.Modifier.html;
*/
public class BaseJavins extends Panel implements RemoteSignalSource
                                                ,LocalSignalSource
                                                ,SignalModifier
                                                ,Serializable
                                                ,Runnable
                                                ,MouseListener
                                                ,MouseMotionListener  {

    //----------------------- RemoteSignalSourgce ------------------------------------
    protected String            sourceURL              = "";
    protected int               sourcePort             = 3000;
    protected transient Socket  remoteSocket;

    //----------------------- LocalSignalSourgce -------------------------------------
    protected String            localSignalSourceClass = "";
    protected LocalSignalSource localSignalSource      = null;

    //-----------------------        Server      -------------------------------------
    protected boolean           server                 = false;
    protected int               serverPort             = 3000;

    //-----------------------  SignalModifier    -------------------------------------
    protected Modifier          modifier;
    protected String            signalModifierClass    = "";

    //-----------------------     General        -------------------------------------
    protected transient SignalPair        signalPair;
    protected transient SignalInputStream sigIn;
    protected String            componentName          = "BaseJavins";
    protected transient Beans   testBean               = new Beans();
    protected boolean           allwaysActive          = false;
    protected boolean           conected               = false;

    protected Label             ltitle                 = new Label(getComponentName());
    protected AlertLabel        alertLabel             = new AlertLabel();
    protected transient Thread            serverThread           = null;
    protected JavinsDispecher   javinsDispecher        = null;
    protected ActionDisplayer   actDisplayer           = null;

    public BaseJavins() {
        super(new BorderLayout(5,5));
        setFont(new Font("Helvetica", Font.PLAIN, 10));
        setBackground(SystemColor.control);
        setForeground(Color.black);

        ltitle.setForeground(SystemColor.activeCaptionText);
        ltitle.setBackground(SystemColor.activeCaption);
        ltitle.setFont(new Font("Helvetica", Font.BOLD, 10));
        ltitle.setAlignment(1);
        add(ltitle, BorderLayout.NORTH);
        add(alertLabel, BorderLayout.SOUTH);
        initJavins();
    }

    public void initJavins() {
        alertLabel.setMessage(false, "Initializing");
        addMouseListener(this);
        addMouseMotionListener(this);
        otherInit();
        if (!testBean.isDesignTime() ||
             allwaysActive ) {
                initializeConections();
             }

    }


    /**
    *   Folosita pentru a permite adaugarea usoara de cod in metoda de initializare
    * prin simpla supradefinire
    */
    public void otherInit () {
    }

    /**
    *   Se initilizeaza toate conexiunile. Pasi urmariti sunt:
    *       1. Se verifica daca nu s-a asignat un obiect LocalSignalSource, la initializare
    *       2. in caz ca nu, se verifica daca nu s-a faut in mediul vizual, prin specificarea
    *           numelui clasei
    *       3. In caz ca nu se vede daca este vorba de o conectiune remote
    */
    public void initializeConections() {
        alertLabel.setMessage(false, "Initialize Conections");
                                                           // cautam mai inti daca se doreste
                                                           //o conexiune locala
        if (getLocalSignalSource() == null) {
            if (getLocalSignalSourceClass() == "") {
                                                           // e clar ca nu avem deci cautam remote
                if (getSourceURL() != "") {
                                                           // am prins-o, remote
                    connectRemoteSignalSource();
                    if (remoteSocket != null)
                            signalPair = getRemoteSignalPair();
                    if (signalPair == null) {
                            alertLabel.setMessage(true, "R.C.Failed");
                            return;
                    }
                    alertLabel.setMessage(false, getSourceURL() + ":" + getSourcePort());
                    conected =true;
                }
                else  alertLabel.setMessage(false, "Javins not connected");
            }
            else {                                        // s-a setat in builder o conexiune locala
                   localSignalSource = getLocalSignalSourceInstance();
                   signalPair = getLocalSignalPair();
                   if (signalPair == null) {
                            alertLabel.setMessage(true, "L.C.Failed");
                            return;
                   }
                   alertLabel.setMessage(false, "Local: " + localSignalSource.getClass().getName());
                   conected =true;
            }
        }
        else {
             signalPair = getLocalSignalPair();
             if (signalPair == null) {
                   alertLabel.setMessage(true, "L.C.Failed");
                   return;
             }
             alertLabel.setMessage(false, "Local: " +
                                  (localSignalSource.getClass().getName()).substring(
                                  localSignalSource.getClass().getName().lastIndexOf(".")+1));

        }

        if (signalPair != null) {
            sigIn = modifySignal(signalPair);
        }
        startServer();
    }


    /**
    * Pentru a putea serializa componenta, trebuie mai intii inchise toate conexiunile
    */
    public void closeConections() {
        if (signalPair != null || remoteSocket != null) {
            try {
                 conected = false;
                 if (sigIn != null) sigIn.close();
                 if (signalPair.signalIn != null) signalPair.signalIn.close();
                 if (signalPair.signalOut!= null) signalPair.signalOut.close();
                 if (remoteSocket != null) remoteSocket.close();
                 stopServer();
            } catch(Exception e) {
                System.err.println("Problems at closing conections!");
                e.printStackTrace();
                System.err.println("Forcing all to be null !");
                sigIn = null;
                signalPair.signalIn = null;
                signalPair.signalOut = null;
            }
         }
    }

    public void setConected(boolean connected) {
        this.conected = connected;
    }

    public void run() {
       try {
             while (conected) {
                if (sigIn != null ) readSignal(sigIn) ;
                else System.out.println("Null signalIn ");
                if (signalPair.signalOut != null )
                    writeSignal(signalPair.signalOut);

             }

       }catch (Exception e) {
            e.printStackTrace();
            System.err.println("Problems at signal read ! ");
            alertLabel.setMessage(true, "R.C.Lost");
       }
    }

    /**
    * In caz ca componenta este si un server de semnal, atunci i-l pornim
    */
    public void startServer() {
        if (isServer()) {
            javinsDispecher = new JavinsDispecher (this);
            serverThread = new Thread (javinsDispecher);
            System.out.println("Prepair to start server ");
            serverThread.start();
        }
    }

    /**
    * Daca este server , atunci la serializare trebuie oprit, pentru a putea
    * serializa componenta
    */
    public void stopServer() {
        if (javinsDispecher != null) {
                javinsDispecher.stopServer();
                serverThread.stop();
                System.out.println(getComponentName() + " server activity stoped !");
        }
    }

    public void readSignal(SignalInputStream mySignalIn) throws IOException{
        System.out.println("Nu e unde trebuie");
        System.out.println(mySignalIn.readByte());
    }

    public void readSignalFServer(DataInputStream mySignalIn) throws IOException{
        System.out.println(mySignalIn.readByte());
    }

    public boolean writeSignal(DataOutputStream mySignalOut) throws IOException{
        mySignalOut.writeByte(0);
        return false;
    }

    public boolean writeSignal2Server(DataOutputStream mySignalOut) throws IOException{
        mySignalOut.writeByte(0);
        return false;
    }

    private void writeObject (ObjectOutputStream oos) throws IOException{
        oos.defaultWriteObject();
    }

    private void readObject (ObjectInputStream ois) throws IOException, ClassNotFoundException {
        ois.defaultReadObject();
        localSignalSourceClass = "";
        initializeConections();
    }


    ///////////////////////////////////////////////////////////////////////////////////////
    //                        * Se seteaza proprietatile *                               //
    ///////////////////////////////////////////////////////////////////////////////////////

    public void setComponentName (String componentName) {
        this.componentName = componentName;
        ltitle.setText(componentName);
    }

    public String getComponentName () {
        return this.componentName;
    }

    public void setSourceURL (String sourceURL) {
        this.sourceURL = sourceURL;
    }

    public String getSourceURL () {
        return this.sourceURL;
    }

    public void setSourcePort (int sourcePort) {
        this.sourcePort = sourcePort;
    }

    public int getSourcePort () {
        return this.sourcePort;
    }

    public void connectRemoteSignalSource() {
        if (   sourceURL  != null
            && sourceURL  != ""
            && sourcePort != -1 ) {
             try {
                remoteSocket = new Socket (
                                     InetAddress.getByName( sourceURL)
                                   , sourcePort);
             } catch (Exception ex) {
                System.err.println (  "Problem Conection to host : "
                                    + sourceURL  + " on port : "
                                    + sourcePort + " failed !");
                ex.printStackTrace();
                alertLabel.setMessage(true, "R.C. Failure");
                System.err.println ( "remoteSocket initialized to -- NULL -- !");
             }
        }
        else remoteSocket = null;
    }

    public SignalPair getRemoteSignalPair() {
        SignalPair sp =  new SignalPair();

        if (remoteSocket != null) {
              try {
                sp.signalIn = new DataInputStream (remoteSocket.getInputStream());
                sp.signalOut= new DataOutputStream(remoteSocket.getOutputStream());

              } catch (Exception ex) {
                    System.err.println( "Could not create SignaPair in : "
                                       + this.getClass().getName()
                                       + " initializing whith -- NULL -- ! ");
                    ex.printStackTrace();
                    sp = null;
                    return null;
               }
        }
        else sp = null;
        return sp;
    }

    public void setLocalSignalSourceClass (String localSignalSourceClass) {
        this.localSignalSourceClass = localSignalSourceClass;
    }

    public String getLocalSignalSourceClass () {
        return this.localSignalSourceClass;
    }

    public LocalSignalSource getLocalSignalSourceInstance () {
        String className = getLocalSignalSourceClass();
        if (className != "" && className != null && localSignalSource == null) {
            try {
                  Class localSignalSClass = Class.forName(className);

                  LocalSignalSource lss = (LocalSignalSource)
                                           localSignalSClass.newInstance();
                  setLocalSignalSource(lss);
                  return lss;
            } catch (ClassNotFoundException ex) {
                    System.err.println("Problems in :"           +
                                       this.getClass().getName() +
                                       " at getLocalSignalSource -> " +
                                       className  + " not found !");
                    ex.printStackTrace();
                    System.err.println("LocalSignalSource initializet to -> NULL <-  !");
                    return null;
            }
              catch (Exception ex) {
                    System.err.println("Problems in :"           +
                                       this.getClass().getName() +
                                       " at getLocalSignalSource -> " +
                                       className  + " can not be initialized !");
                    ex.printStackTrace();
                    System.err.println("LocalSignalSource initializet to -> NULL <-  !");
                    return null;
              }
        }
        else return null;
    }

    public void setLocalSignalSource ( LocalSignalSource localSignalSource) {
        this.localSignalSource = localSignalSource;
    }

    public LocalSignalSource getLocalSignalSource () {
        if (localSignalSource == null)
        return null;
        else return this.localSignalSource;
    }

    public SignalPair getLocalSignalPair () {
        LocalSignalSource lss = getLocalSignalSource();

        if (lss != null) return lss.getLocalSignalPair();
        else return null;
    }

    public void setAllwaysActive (boolean allwaysActive) {
        this.allwaysActive = allwaysActive;
        if (allwaysActive) {
                            initializeConections();
                            run();
        }
    }

    public boolean isAllwaysActive() {
        return this.allwaysActive;
    }

    public void setServer (boolean server) {
        this.server = server;
        // --- se porneste
    }

    public boolean isServer() {
        return this.server;
    }

    public void setServerPort(int serverPort) {
        this.serverPort = serverPort;
    }

    public int getServerPort () {
        return this.serverPort;
    }

    public void setSignalModifierClass (String signalModifierClass) {
        this.signalModifierClass = signalModifierClass;
    }

    public String getSignalModifierClass () {
        return this.signalModifierClass;
    }

    public void setSignalModifier ( Modifier modifier) {
        this.modifier = modifier;
    }

    public Modifier getSignalModifier () {
        return this.modifier;
    }

    public void instantiateSignalModifierClass () {
        // nimic inca
    }

    public SignalInputStream modifySignal (SignalPair signalPair) {
        SignalInputStream signalInputStream;
        try { signalInputStream = new SignalInputStream(signalPair.signalIn, getSignalModifier());
              return signalInputStream;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public void setActionDisplayer(ActionDisplayer actdis) {
        this.actDisplayer = actdis;
    }

    public void communicate(String s) {
        if (actDisplayer != null) {
                actDisplayer.communicate(getComponentName(), s);
        }
    }

    ///////////////////////////////////////////////////////////////////////////////////////
    //                        * Se seteaza diverse metode *                              //
    ///////////////////////////////////////////////////////////////////////////////////////

    protected void finalize() throws Throwable {
            //closeConections();
            //stopServer();
    }

    public void mouseClicked (MouseEvent me) {;}

    public void mouseEntered (MouseEvent me) {;}

    public void mouseExited  (MouseEvent me) {;}

    public void mousePressed (MouseEvent me) {;}

    public void mouseReleased(MouseEvent me) {;}

    public void mouseDragged (MouseEvent me) {;}

    public void mouseMoved   (MouseEvent me) {;}

    ///////////////////////////////////////////////////////////////////////////////////////
    //                        *     Metoda numai de test  *                              //
    ///////////////////////////////////////////////////////////////////////////////////////

    public static void main (String args[]) {
        BaseJavins bj = new BaseJavins();
        Frame f = new Frame(bj.getComponentName() + " -> Test <-");
        f.setLayout(new BorderLayout());
        f.add(bj);
        f.setSize(bj.getSize().width, bj.getSize().height);
        f.pack();
        f.show();
        System.runFinalizersOnExit(true);
        new WindowCloser(f, true);

        bj.setSourceURL("clona");
        bj.setSourcePort(3000);
        bj.initializeConections();
        bj.run();


    }
}
