/**********************************************************************************
 * ClassName  :   LocalSignalSource
 * Date       :   17, Mai  1998
 * Autor      :   Remus Pereni [RP]
 * Description:   Vezi mai jos
 **********************************************************************************/


package JaRPE.Javins;

import java.io.*;

/**
* LocalSignalSource
*
* Este interfata care garanteaza o sursa de date (stream de date)
* dintr-o sursa locala. Poate fi conceput ca un plug-in care se
* cupleaza intr-un mediu vizual sau nu cu o componenta (Javins) gata
* si asigura semnalul pentru aceasta.
*   Are menirea de a asigura cuplarea unei componente cu uni dispozitiv
* hardware. Si separarea in acest fel a componentelor stabile de cele
* care se modifica. Deci nu mai este nevoie de scrierea unei componente
* noi care sa stie sa citeasca un anumit dispozitiv, ci se foloseste
* o componenta standard, si se paricularizeaza numai o clasa care stie
* sa lucreze cu acel dispozitiv si sa furnizeze prin aceasta interfata
* SignalPair-urile, necesare Javins-ului, pentru a asigura comunicatia
* bidirectionala cu acel dispozitiv
*
* @author   Remus Pereni
* @version  0.01, 17 May 1998 01:30 AM
*/

public interface LocalSignalSource {

    /**
    *   Pastreaza numele clasei din care se instantiaza o sursa locala
    */
    String              localSignalSourceClass = "";
    LocalSignalSource   localSignalSource      = null;

    /**
    *   Asigura metodele standard pentru un Bean de setare si preluare
    *   a numelui clasei din care se va instantia obiectul
    *   @param localSignalSourceClass  - numele clasei din care se va
    *                                    instantia obiectul
    */
    public void setLocalSignalSourceClass(String localSignalSourceClass);

    /**
    *   Asigura metodele standard pentru un Bean de setare si preluare
    *   a numelui clasei din care se va instantia obiectul
    *   @return localSignalSourceClass  - numele clasei din care se va
    *                                     instantia obiectul
    */
    public String getLocalSignalSourceClass();


    public void setLocalSignalSource (LocalSignalSource localSignalSource);


    public LocalSignalSource getLocalSignalSource ();
    /**
    *   Realizeaza instantierea obiectului.
    * Din numele clasei setat, se creaza clasa si se instantiaza
    *   @return LocalSignalSource - instanta generatorului de semnal local
    */
    public LocalSignalSource getLocalSignalSourceInstance();

    /**
    *  Din instanta de generator de semnal local, data initial ca si nume de
    * clasa, se culege perechea de streamuri de data (DataInputStream,
    * DataOutputStream), pentru comunicatia bidirectionala
    *   @return SignalPair - Cele doua streamuri (In/Out de date)
    */
    public SignalPair getLocalSignalPair();
}

