
package JaRPE.Javins;


import java.io.*;

public class SignalInputStream extends FilterInputStream implements DataInput {

    private Modifier mod = null;

    public SignalInputStream(InputStream in) {
    	super(in);
    }

    public SignalInputStream(InputStream in, Modifier mod) {
    	super(in);
    	this.mod = mod;
    }

    public final int read(byte b[]) throws IOException {
    	return in.read(b, 0, b.length);
    }

    public final int read(byte b[], int off, int len) throws IOException {
	    return in.read(b, off, len);
    }

    public final void readFully(byte b[]) throws IOException {
    	readFully(b, 0, b.length);
    }
    public final void readFully(byte b[], int off, int len) throws IOException {
    	InputStream in = this.in;
    	int n = 0;
    	while (n < len) {
    	    int count = in.read(b, off + n, len - n);
    	    if (count < 0)
    		throw new EOFException();
    	    n += count;
    	}
    }

    public final int skipBytes(int n) throws IOException {
    	InputStream in = this.in;
    	for (int i = 0 ; i < n ; i += (int)in.skip(n - i));
    	return n;
    }

    public final boolean readBoolean() throws IOException {
    	int ch = in.read();
    	if (ch < 0)
    	    throw new EOFException();
    	if (mod == null) return (ch != 0);
    	else return mod.modifyBoolean(ch != 0);
    }

    public final byte readByte() throws IOException {
    	int ch = in.read();
    	if (ch < 0)
    	    throw new EOFException();
    	if (mod == null) return (byte)(ch);
    	else return mod.modifyByte((byte)(ch));
    }

    public final int readUnsignedByte() throws IOException {
    	int ch = in.read();
    	if (ch < 0)
    	    throw new EOFException();
    	if (mod == null) return ch;
    	else return mod.modifyInt(ch);
    }

    public final short readShort() throws IOException {
    	InputStream in = this.in;
    	int ch1 = in.read();
    	int ch2 = in.read();
    	if ((ch1 | ch2) < 0)
    	     throw new EOFException();
        if (mod == null) return (short)((ch1 << 8) + (ch2 << 0));
        else return mod.modifyShort((short)((ch1 << 8) + (ch2 << 0)));
    }

    public final int readUnsignedShort() throws IOException {
    	InputStream in = this.in;
    	int ch1 = in.read();
    	int ch2 = in.read();
    	if ((ch1 | ch2) < 0)
    	     throw new EOFException();
        if (mod == null) return (ch1 << 8) + (ch2 << 0);
        else return mod.modifyInt((ch1 << 8) + (ch2 << 0));
    }

    public final char readChar() throws IOException {
    	InputStream in = this.in;
    	int ch1 = in.read();
    	int ch2 = in.read();
    	if ((ch1 | ch2) < 0)
    	     throw new EOFException();
    	if (mod == null) return (char)((ch1 << 8) + (ch2 << 0));
    	else  return mod.modifyChar((char)((ch1 << 8) + (ch2 << 0)));
    }

    public final int readInt() throws IOException {
    	InputStream in = this.in;
    	int ch1 = in.read();
    	int ch2 = in.read();
    	int ch3 = in.read();
    	int ch4 = in.read();
    	if ((ch1 | ch2 | ch3 | ch4) < 0)
    	     throw new EOFException();
        if (mod == null) return ((ch1 << 24) + (ch2 << 16) + (ch3 << 8) + (ch4 << 0));
        else return mod.modifyInt(((ch1 << 24) + (ch2 << 16) + (ch3 << 8) + (ch4 << 0)));
    }

    public final long readLong() throws IOException {
    	InputStream in = this.in;
        if (mod == null) return ((long)(readInt()) << 32) + (readInt() & 0xFFFFFFFFL);
        else return mod.modifyLong(((long)(readInt()) << 32) + (readInt() & 0xFFFFFFFFL));
    }

    public final float readFloat() throws IOException {
        if (mod == null) return Float.intBitsToFloat(readInt());
        else return mod.modifyFloat(Float.intBitsToFloat(readInt()));
    }

    public final double readDouble() throws IOException {
        if (mod == null) return Double.longBitsToDouble(readLong());
        else return mod.modifyDouble(Double.longBitsToDouble(readLong()));
    }

    private char lineBuffer[];

    public final String readLine() throws IOException {
    	InputStream in = this.in;
    	char buf[] = lineBuffer;

    	if (buf == null) {
    	    buf = lineBuffer = new char[128];
    	}

    	int room = buf.length;
    	int offset = 0;
    	int c;

    loop:	while (true) {
    	    switch (c = in.read()) {
    	      case -1:
    	      case '\n':
    		break loop;

    	      case '\r':
    		int c2 = in.read();
    		if (c2 != '\n') {
    		    if (!(in instanceof PushbackInputStream)) {
    			in = this.in = new PushbackInputStream(in);
    		    }
    		    ((PushbackInputStream)in).unread(c2);
    		}
    		break loop;

    	      default:
    		if (--room < 0) {
    		    buf = new char[offset + 128];
    		    room = buf.length - offset - 1;
    		    System.arraycopy(lineBuffer, 0, buf, 0, offset);
    		    lineBuffer = buf;
    		}
    		buf[offset++] = (char) c;
    		break;
    	    }
    	}
    	if ((c == -1) && (offset == 0)) {
    	    return null;
    	}

    	if (mod == null) return String.copyValueOf(buf, 0, offset);
    	else return mod.modifyString(String.copyValueOf(buf, 0, offset));
    }

    public final String readUTF() throws IOException {
        return readUTF(this);
    }

    public final static String readUTF(DataInput in) throws IOException {
            int utflen = in.readUnsignedShort();
            char str[] = new char[utflen];
    	int count = 0;
    	int strlen = 0;
    	while (count < utflen) {
    	    int c = in.readUnsignedByte();
    	    int char2, char3;
    	    switch (c >> 4) {
    	        case 0: case 1: case 2: case 3: case 4: case 5: case 6: case 7:
    		    // 0xxxxxxx
    		    count++;
    		    str[strlen++] = (char)c;
    		    break;
    	        case 12: case 13:
    		    // 110x xxxx   10xx xxxx
    		    count += 2;
    		    if (count > utflen)
    			throw new UTFDataFormatException();
    		    char2 = in.readUnsignedByte();
    		    if ((char2 & 0xC0) != 0x80)
    			throw new UTFDataFormatException();
    		    str[strlen++] = (char)(((c & 0x1F) << 6) | (char2 & 0x3F));
    		    break;
    	        case 14:
    		    // 1110 xxxx  10xx xxxx  10xx xxxx
    		    count += 3;
    		    if (count > utflen)
    			throw new UTFDataFormatException();
    		    char2 = in.readUnsignedByte();
    		    char3 = in.readUnsignedByte();
    		    if (((char2 & 0xC0) != 0x80) || ((char3 & 0xC0) != 0x80))
    			throw new UTFDataFormatException();
    		    str[strlen++] = (char)(((c & 0x0F) << 12) |
    					   ((char2 & 0x3F) << 6) |
    					   ((char3 & 0x3F) << 0));
    		    break;
    	        default:
    		    // 10xx xxxx,  1111 xxxx
    		    throw new UTFDataFormatException();
    		}
    	}
            return new String(str, 0, strlen);

        }
}
