/**********************************************************************************
 * ClassName  :   JavinstWrapper
 * Date       :   17, May  1998
 * Autor      :   Remus Pereni [RP]
 * Description:   Vezi mai jos
 **********************************************************************************/

package JaRPE;

import java.io.Serializable;


/**
* JavinstWrapper
*   Implementeaza un wrapper pentru obiectele de interfata,
* Javinst-uri (Java Vitual Instruments), asta pentru a asigura un
* model comun pentru exportarea datelor, si anume datorita pierderi
* detaliilor legate de clasa la serializare. Deci este o structura ce
* contine numele clasei, obiectul si o descriere a ei
* <p>  Teoretic clasa respecta specificatiile de implementare a Beanurilor
*
* @author   Remus Pereni
* @version  0.01, 17 May 1998 01:30 AM
* @see      java.io.Serializable;
*/
public class JavinstWrapper implements Serializable {
    private Object javinstObject;
    private String javinstClass;
    private String description;

    /** Constructorul clasei, fara parametrii conform tiparului Beanurilor
     */
    public JavinstWrapper() {
    }

    /** Constructor cu initializarea elementelor esentiale (strictul necesar)
     * @param javinstObject    Instanta instrumentului
     * @param javinstClass     Numele clasei obiectului, necesar pentru
     *                         refacerea corecta a lui dupa serializare
     */
    public JavinstWrapper(Object javinstObject, String javinstClass) {
        this.javinstObject = javinstObject;
        this.javinstClass  = javinstClass;
    }

    /** Constructor cu initializarea completa a elemntelor
     * @param javinstObject    Instanta instrumentului
     * @param javinstClass     Numele clasei obiectului, necesar pentru
     *                         refacerea corecta a lui dupa serializare
     * @param descr            O scurta descriere a obiectului, scopul
     *                         fiind pur informational
     */
    public JavinstWrapper(Object javinstObject, String javinstClass,
                          String description ) {
        this.javinstObject = javinstObject;
        this.javinstClass  = javinstClass;
        this.description   = description;
    }

    /** Metoda standard pentru preluarea obiectului
     * @return javinstObject    instanta obiectului
     */
    public Object getJavinstObject() {
        return this.javinstObject;
    }

    /** Metoda standard pentru preluarea classei care indentifica obiectul
     * @return javinstClass    class din care a fost instantiat obiectul
     */
    public String getJavinstClass() {
        return this.javinstClass;
    }

    /** Metoda standard pentru preluare descrierii
     * @ return description     O informatie oarecare cu caracter customizat
     */
    public String getDescription() {
        return this.description;
    }

}